from flask import request

from ocs.api.views.base import BaseView

from ocs.conf import build_object_from_conf
from ocs.api.clients import AuthAPIClient, ComputeAPIClient
from ocs.api.exceptions import NotFound


CONFIGURATION_NAME = 'ocs/api-metadata'


def client_ip():
    """ Returns the client IP address.
    """
    return request.remote_addr


class MetadataAPIBaseView(BaseView):

    privileged_account_api = None
    privileged_compute_api = None

    def before_request(self, *args, **kwargs):
        super(MetadataAPIBaseView, self).before_request(*args, **kwargs)

        self.privileged_account_api = build_object_from_conf(
            AuthAPIClient, {
                'auth_token': (CONFIGURATION_NAME,
                               'api-metadata.api-account.token'),

                'base_url': (CONFIGURATION_NAME,
                             'api-metadata.api-account.url',
                             None),

                'verify_ssl': (CONFIGURATION_NAME,
                               'api-metadata.api-account.verify_ssl',
                               True),
            }
        )

        self.privileged_compute_api = build_object_from_conf(
            ComputeAPIClient, {
                'auth_token': (CONFIGURATION_NAME,
                               'api-metadata.api-compute.token'),

                'base_url': (CONFIGURATION_NAME,
                             'api-metadata.api-compute.url',
                             None),

                'verify_ssl': (CONFIGURATION_NAME,
                               'api-metadata.api-compute.verify_ssl',
                               True),
            }
        )

    def _get_server_by_ip(self, ip_addr):
        response = self.privileged_compute_api.query() \
                       .servers.get(private_ip=ip_addr)

        servers = response.get('servers', [])

        if len(servers) == 0:
            raise NotFound('No server with ip %s' % ip_addr)
        if len(servers) > 1:
            raise NotFound('More than one server with ip %s (wtf?)' % ip_addr)

        return servers[0]
